# Generated by Django 2.1.5 on 2019-02-19 20:02

from django.db import migrations


def forward(apps, schema_editor):
    Client = apps.get_model('django_keycloak', 'Client')
    for client in Client.objects.filter(service_account__isnull=False):
        client.service_account_profile = client.service_account.oidc_profile
        client.save()


def backward(apps, schema_editor):
    Client = apps.get_model('django_keycloak', 'Client')
    for client in Client.objects.filter(service_account_profile__isnull=False):
        client.service_account = client.service_account_profile.user
        client.save()


class Migration(migrations.Migration):

    dependencies = [
        ('django_keycloak', '0004_client_service_account_profile'),
    ]

    operations = [
        migrations.RunPython(forward, backward),
    ]



import mock

from datetime import datetime

from django.test import TestCase
from freezegun import freeze_time
from keycloak.openid_connect import KeycloakOpenidConnect

from django_keycloak.factories import OpenIdConnectProfileFactory
from django_keycloak.tests.mixins import MockTestCaseMixin

import django_keycloak.services.oidc_profile


class ServicesKeycloakOpenIDProfileGetActiveAccessTokenTestCase(
        MockTestCaseMixin, TestCase):

    def setUp(self):
        self.oidc_profile = OpenIdConnectProfileFactory(
            access_token='access-token',
            expires_before=datetime(2018, 3, 5, 1, 0, 0),
            refresh_token='refresh-token',
            refresh_expires_before=datetime(2018, 3, 5, 2, 0, 0)
        )
        self.oidc_profile.realm.client.openid_api_client = mock.MagicMock(
            spec_set=KeycloakOpenidConnect)
        self.oidc_profile.realm.client.openid_api_client.refresh_token\
            .return_value = {
                'access_token': 'new-access-token',
                'expires_in': 600,
                'refresh_token': 'new-refresh-token',
                'refresh_expires_in': 3600
            }

    @freeze_time('2018-03-05 00:59:00')
    def test_not_expired(self):
        """
        Case: access token get fetched and is not yet expired
        Expected: current token is returned
        """
        access_token = django_keycloak.services.oidc_profile\
            .get_active_access_token(oidc_profile=self.oidc_profile)

        self.assertEqual(access_token, 'access-token')
        self.assertFalse(
            self.oidc_profile.realm.client.openid_api_client.refresh_token
                .called
        )

    @freeze_time('2018-03-05 01:01:00')
    def test_expired(self):
        """
        Case: access token get requested but current one is expired
        Expected: A new one get requested
        """
        access_token = django_keycloak.services.oidc_profile \
            .get_active_access_token(oidc_profile=self.oidc_profile)

        self.assertEqual(access_token, 'new-access-token')
        self.oidc_profile.realm.client.openid_api_client.refresh_token\
            .assert_called_once_with(refresh_token='refresh-token')

        self.oidc_profile.refresh_from_db()
        self.assertEqual(self.oidc_profile.access_token, 'new-access-token')
        self.assertEqual(self.oidc_profile.expires_before,
                         datetime(2018, 3, 5, 1, 11, 0))
        self.assertEqual(self.oidc_profile.refresh_token, 'new-refresh-token')
        self.assertEqual(self.oidc_profile.refresh_expires_before,
                         datetime(2018, 3, 5, 2, 1, 0))



# Generated by Django 2.0.2 on 2018-03-15 21:15

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Client',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True,
                                        serialize=False, verbose_name='ID')),
                ('client_id', models.CharField(max_length=255)),
                ('secret', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Nonce',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True,
                                        serialize=False, verbose_name='ID')),
                ('state', models.UUIDField(default=uuid.uuid4, unique=True)),
                ('redirect_uri', models.CharField(max_length=255)),
                ('next_path', models.CharField(max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='OpenIdConnectProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True,
                                        serialize=False, verbose_name='ID')),
                ('access_token', models.TextField(null=True)),
                ('expires_before', models.DateTimeField(null=True)),
                ('refresh_token', models.TextField(null=True)),
                ('refresh_expires_before', models.DateTimeField(null=True)),
                ('sub', models.CharField(max_length=255, unique=True)),
            ],
            options={
                'abstract': False,
                'swappable': 'KEYCLOAK_OIDC_PROFILE_MODEL',
            },
        ),
        migrations.CreateModel(
            name='Realm',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True,
                                        serialize=False, verbose_name='ID')),
                ('name', models.CharField(
                    help_text='Name as known on the Keycloak server. This '
                              'name is used in the API paths of this Realm.',
                    max_length=255, unique=True)),
                ('_certs', models.TextField()),
                ('_well_known_oidc', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True,
                                        serialize=False, verbose_name='ID')),
                ('reference', models.CharField(max_length=50)),
                ('client', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='roles', to='django_keycloak.Client')),
                ('permission', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    to='auth.Permission')),
            ],
        ),
        migrations.CreateModel(
            name='Server',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True,
                                        serialize=False, verbose_name='ID')),
                ('url', models.CharField(max_length=255)),
                ('internal_url', models.CharField(
                    blank=True,
                    help_text='URL on internal netwerk calls. For example '
                              'when used with Docker Compose. Only supply '
                              'when internal calls should go to a different '
                              'url as the end-user will communicate with.',
                    max_length=255, null=True)),
            ],
        ),
        migrations.AddField(
            model_name='realm',
            name='server',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='realms', to='django_keycloak.Server'),
        ),
        migrations.AddField(
            model_name='openidconnectprofile',
            name='realm',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='openid_profiles', to='django_keycloak.Realm'),
        ),
        migrations.AddField(
            model_name='openidconnectprofile',
            name='user',
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='oidc_profile', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='client',
            name='realm',
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='client', to='django_keycloak.Realm'),
        ),
        migrations.AddField(
            model_name='client',
            name='service_account',
            field=models.OneToOneField(
                null=True, on_delete=django.db.models.deletion.CASCADE,
                related_name='keycloak_client', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='role',
            unique_together={('client', 'permission')},
        ),
    ]



# Generated by Django 2.1.5 on 2019-02-19 13:23

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_keycloak', '0003_auto_20190204_1949'),
    ]

    operations = [
        migrations.AddField(
            model_name='client',
            name='service_account_profile',
            field=models.OneToOneField(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.KEYCLOAK_OIDC_PROFILE_MODEL
            ),
        ),
    ]



from __future__ import unicode_literals

import logging

from django.core.management.base import BaseCommand

from django_keycloak.models import Realm

import django_keycloak.services.realm

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    def handle(self, *args, **options):
        for realm in Realm.objects.all():
            django_keycloak.services.realm.refresh_well_known_oidc(realm=realm)
            django_keycloak.services.realm.refresh_certs(realm=realm)
            logger.debug('Refreshed: {}'.format(realm))



from datetime import timedelta

import logging

from django.apps import apps as django_apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.contrib.auth import get_user_model
from django.db import transaction
from django.utils import timezone
from django.utils.module_loading import import_string
from keycloak.exceptions import KeycloakClientError

from django_keycloak.services.exceptions import TokensExpired
from django_keycloak.remote_user import KeycloakRemoteUser


import django_keycloak.services.realm

logger = logging.getLogger(__name__)


def get_openid_connect_profile_model():
    """
    Return the OpenIdConnectProfile model that is active in this project.
    """
    try:
        return django_apps.get_model(settings.KEYCLOAK_OIDC_PROFILE_MODEL,
                                     require_ready=False)
    except ValueError:
        raise ImproperlyConfigured(
            "KEYCLOAK_OIDC_PROFILE_MODEL must be of the form "
            "'app_label.model_name'")
    except LookupError:
        raise ImproperlyConfigured(
            "KEYCLOAK_OIDC_PROFILE_MODEL refers to model '%s' that has not "
            "been installed" % settings.KEYCLOAK_OIDC_PROFILE_MODEL)


def get_remote_user_model():
    """
    Return the User model that is active in this project.
    """
    if not hasattr(settings, 'KEYCLOAK_REMOTE_USER_MODEL'):
        # By default return the standard KeycloakRemoteUser model
        return KeycloakRemoteUser

    try:
        return import_string(settings.KEYCLOAK_REMOTE_USER_MODEL)
    except ImportError:
        raise ImproperlyConfigured(
            "KEYCLOAK_REMOTE_USER_MODEL refers to non-existing class"
        )


def get_or_create_from_id_token(client, id_token):
    """
    Get or create OpenID Connect profile from given id_token.

    :param django_keycloak.models.Client client:
    :param str id_token:
    :rtype: django_keycloak.models.OpenIdConnectProfile
    """
    issuer = django_keycloak.services.realm.get_issuer(client.realm)

    id_token_object = client.openid_api_client.decode_token(
        token=id_token,
        key=client.realm.certs,
        algorithms=client.openid_api_client.well_known[
            'id_token_signing_alg_values_supported'],
        issuer=issuer
    )

    return update_or_create_user_and_oidc_profile(
        client=client, id_token_object=id_token_object)


def update_or_create_user_and_oidc_profile(client, id_token_object):
    """

    :param client:
    :param id_token_object:
    :return:
    """

    OpenIdConnectProfileModel = get_openid_connect_profile_model()

    if OpenIdConnectProfileModel.is_remote:
        oidc_profile, _ = OpenIdConnectProfileModel.objects.\
            update_or_create(
                sub=id_token_object['sub'],
                defaults={
                    'realm': client.realm
                }
            )

        UserModel = get_remote_user_model()
        oidc_profile.user = UserModel(id_token_object)

        return oidc_profile

    with transaction.atomic():
        UserModel = get_user_model()
        email_field_name = UserModel.get_email_field_name()
        user, _ = UserModel.objects.update_or_create(
            username=id_token_object['sub'],
            defaults={
                email_field_name: id_token_object.get('email', ''),
                'first_name': id_token_object.get('given_name', ''),
                'last_name': id_token_object.get('family_name', '')
            }
        )

        oidc_profile, _ = OpenIdConnectProfileModel.objects.update_or_create(
            sub=id_token_object['sub'],
            defaults={
                'realm': client.realm,
                'user': user
            }
        )

    return oidc_profile


def get_remote_user_from_profile(oidc_profile):
    """

    :param oidc_profile:
    :return:
    """

    try:
        userinfo = oidc_profile.realm.client.openid_api_client.userinfo(
            token=oidc_profile.access_token
        )
    except KeycloakClientError:
        return None

    # Get the user from the KEYCLOAK_REMOTE_USER_MODEL in the settings
    UserModel = get_remote_user_model()

    # Create the object of type UserModel from the constructor of it's class
    # as the included details can vary per model
    user = UserModel(userinfo)

    return user


def update_or_create_from_code(code, client, redirect_uri):
    """
    Update or create an user based on an authentication code.
    Response as specified in:

    https://tools.ietf.org/html/rfc6749#section-4.1.4

    :param django_keycloak.models.Client client:
    :param str code: authentication code
    :param str redirect_uri
    :rtype: django_keycloak.models.OpenIdConnectProfile
    """

    # Define "initiate_time" before getting the access token to calculate
    # before which time it expires.
    initiate_time = timezone.now()
    token_response = client.openid_api_client.authorization_code(
        code=code, redirect_uri=redirect_uri)

    return _update_or_create(client=client, token_response=token_response,
                             initiate_time=initiate_time)


def update_or_create_from_password_credentials(username, password, client):
    """
    Update or create an user based on username and password.
    Response as specified in:

    https://tools.ietf.org/html/rfc6749#section-4.3.3

    :param str username: the username to authenticate with
    :param str password: the password to authenticate with
    :param django_keycloak.models.Client client:
    :rtype: django_keycloak.models.OpenIdConnectProfile
    """

    # Define "initiate_time" before getting the access token to calculate
    # before which time it expires.
    initiate_time = timezone.now()
    token_response = client.openid_api_client.password_credentials(
        username=username, password=password)

    return _update_or_create(client=client, token_response=token_response,
                             initiate_time=initiate_time)


def _update_or_create(client, token_response, initiate_time):
    """
    Update or create an user based on a token response.

    `token_response` contains the items returned by the OpenIDConnect Token API
    end-point:
     - id_token
     - access_token
     - expires_in
     - refresh_token
     - refresh_expires_in

    :param django_keycloak.models.Client client:
    :param dict token_response:
    :param datetime.datetime initiate_time:
    :rtype: django_keycloak.models.OpenIdConnectProfile
    """
    issuer = django_keycloak.services.realm.get_issuer(client.realm)

    token_response_key = 'id_token' if 'id_token' in token_response \
        else 'access_token'

    token_object = client.openid_api_client.decode_token(
        token=token_response[token_response_key],
        key=client.realm.certs,
        algorithms=client.openid_api_client.well_known[
            'id_token_signing_alg_values_supported'],
        issuer=issuer
    )

    oidc_profile = update_or_create_user_and_oidc_profile(
        client=client,
        id_token_object=token_object)

    return update_tokens(token_model=oidc_profile,
                         token_response=token_response,
                         initiate_time=initiate_time)


def update_tokens(token_model, token_response, initiate_time):
    """
    Update tokens on the OpenID Connect profile

    :param django_keycloak.models.TokenModelAbstract token_model:
    :param dict token_response: response from OIDC token API end-point
    :param datetime.datetime initiate_time: timestamp before the token request
    :rtype: django_keycloak.models.OpenIdConnectProfile
    """
    expires_before = initiate_time + timedelta(
        seconds=token_response['expires_in'])
    refresh_expires_before = initiate_time + timedelta(
        seconds=token_response['refresh_expires_in'])

    token_model.access_token = token_response['access_token']
    token_model.expires_before = expires_before
    token_model.refresh_token = token_response['refresh_token']
    token_model.refresh_expires_before = refresh_expires_before

    token_model.save(update_fields=['access_token',
                                    'expires_before',
                                    'refresh_token',
                                    'refresh_expires_before'])
    return token_model


def get_active_access_token(oidc_profile):
    """
    Give access_token and refresh when required.

    :param django_keycloak.models.KeycloakOpenIDProfile openid_profile:
    :rtype: string
    :raise: django_keycloak.services.exceptions.TokensExpired
    """
    initiate_time = timezone.now()

    if oidc_profile.refresh_expires_before is None \
            or initiate_time > oidc_profile.refresh_expires_before:
        raise TokensExpired()

    if initiate_time > oidc_profile.expires_before:
        # Refresh token
        token_response = oidc_profile.realm.client.openid_api_client\
            .refresh_token(refresh_token=oidc_profile.refresh_token)

        oidc_profile = update_tokens(token_model=oidc_profile,
                                     token_response=token_response,
                                     initiate_time=initiate_time)

    return oidc_profile.access_token


def get_entitlement(oidc_profile):
    """
    Get entitlement.

    http://www.keycloak.org/docs/latest/authorization_services/index.html#_service_entitlement_api

    :param django_keycloak.models.KeycloakOpenIDProfile oidc_profile:
    :rtype: dict
    :return: Decoded RPT
    """
    access_token = get_active_access_token(oidc_profile=oidc_profile)
    if settings.KEYCLOAK_VERSION == 3:
        rpt = oidc_profile.realm.client.authz_api_client.entitlement(
            token=access_token)
        token = rpt['rpt']
    else:
        rpt = oidc_profile.realm.client.openid_api_client.uma_ticket(
            token=access_token,
            audience=oidc_profile.realm.client.client_id
        )
        token = rpt['access_token']

    rpt_decoded = oidc_profile.realm.client.openid_api_client.decode_token(
        token=token,
        key=oidc_profile.realm.certs,
        options={
            'verify_signature': True,
            'exp': True,
            'iat': True,
            'aud': True
        })
    return rpt_decoded


def get_decoded_jwt(oidc_profile):
    """
    :param django_keycloak.models.KeycloakOpenIDProfile oidc_profile:
    :rtype dict
    """

    client = oidc_profile.realm.client

    active_access_token = get_active_access_token(oidc_profile=oidc_profile)

    return client.openid_api_client.decode_token(
        token=active_access_token,
        key=client.realm.certs,
        algorithms=client.openid_api_client.well_known[
            'id_token_signing_alg_values_supported']
    )



# Generated by Django 2.1.5 on 2019-02-04 19:49

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_keycloak', '0002_auto_20180322_2059'),
    ]

    operations = [
        migrations.CreateModel(
            name='RemoteUserOpenIdConnectProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True,
                                        serialize=False, verbose_name='ID')),
                ('access_token', models.TextField(null=True)),
                ('expires_before', models.DateTimeField(null=True)),
                ('refresh_token', models.TextField(null=True)),
                ('refresh_expires_before', models.DateTimeField(null=True)),
                ('sub', models.CharField(max_length=255, unique=True)),
                ('realm', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='openid_profiles',
                    to='django_keycloak.Realm'
                )),
            ],
            options={
                'abstract': False,
                'swappable': 'KEYCLOAK_OIDC_PROFILE_MODEL',
            },
        ),
        migrations.AlterField(
            model_name='exchangedtoken',
            name='oidc_profile',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.KEYCLOAK_OIDC_PROFILE_MODEL
            ),
        ),
    ]



# Generated by Django 2.1.5 on 2019-02-19 20:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_keycloak', '0005_auto_20190219_2002'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='client',
            name='service_account',
        ),
    ]



import logging

from django_keycloak.app_settings import *  # noqa: F403,F401

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.MD5PasswordHasher',
)

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'secret-key'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

LOGIN_URL = 'keycloak_login'
LOGOUT_REDIRECT_URL = 'index'

# Application definition
INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',

    'django_keycloak.apps.KeycloakAppConfig',
]

MIDDLEWARE = [
    'django_keycloak.middleware.BaseKeycloakMiddleware',
]

AUTHENTICATION_BACKENDS = [
    'django.contrib.auth.backends.ModelBackend',
    'django_keycloak.auth.backends.KeycloakAuthorizationCodeBackend',
]

DATABASES = {
  'default': {
    'ENGINE': 'django.db.backends.sqlite3',
    'NAME': ':memory:',
  }
}

logging.disable(logging.CRITICAL)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging

from django.shortcuts import resolve_url

from django_keycloak.services.oidc_profile import get_remote_user_model

try:
    from urllib.parse import urljoin  # noqa: F401
except ImportError:
    from urlparse import urljoin  # noqa: F401

from django.conf import settings
from django.contrib.auth import authenticate, login, logout
from django.core.exceptions import ObjectDoesNotExist
from django.http.response import (
    HttpResponseBadRequest,
    HttpResponseServerError,
    HttpResponseRedirect
)
from django.urls.base import reverse
from django.views.generic.base import (
    RedirectView,
    TemplateView
)

from django_keycloak.models import Nonce
from django_keycloak.auth import remote_user_login


logger = logging.getLogger(__name__)


class Login(RedirectView):

    def get_redirect_url(self, *args, **kwargs):

        nonce = Nonce.objects.create(
            redirect_uri=self.request.build_absolute_uri(
                location=reverse('keycloak_login_complete')),
            next_path=self.request.GET.get('next'))

        self.request.session['oidc_state'] = str(nonce.state)

        authorization_url = self.request.realm.client.openid_api_client\
            .authorization_url(
                redirect_uri=nonce.redirect_uri,
                scope='openid given_name family_name email',
                state=str(nonce.state)
            )

        if self.request.realm.server.internal_url:
            authorization_url = authorization_url.replace(
                self.request.realm.server.internal_url,
                self.request.realm.server.url,
                1
            )

        logger.debug(authorization_url)

        return authorization_url


class LoginComplete(RedirectView):

    def get(self, *args, **kwargs):
        request = self.request

        if 'error' in request.GET:
            return HttpResponseServerError(request.GET['error'])

        if 'code' not in request.GET and 'state' not in request.GET:
            return HttpResponseBadRequest()

        if 'oidc_state' not in request.session \
                or request.GET['state'] != request.session['oidc_state']:
            # Missing or incorrect state; login again.
            return HttpResponseRedirect(reverse('keycloak_login'))

        nonce = Nonce.objects.get(state=request.GET['state'])

        user = authenticate(request=request,
                            code=request.GET['code'],
                            redirect_uri=nonce.redirect_uri)

        RemoteUserModel = get_remote_user_model()
        if isinstance(user, RemoteUserModel):
            remote_user_login(request, user)
        else:
            login(request, user)

        nonce.delete()

        return HttpResponseRedirect(nonce.next_path or '/')


class Logout(RedirectView):

    def get_redirect_url(self, *args, **kwargs):
        try:
            oidc_profile = self.request.user.oidc_profile
        except ObjectDoesNotExist:
            pass
        else:
            self.request.realm.client.openid_api_client.logout(
                oidc_profile.refresh_token
            )
            oidc_profile.access_token = None
            oidc_profile.expires_before = None
            oidc_profile.refresh_token = None
            oidc_profile.refresh_expires_before = None
            oidc_profile.save(update_fields=[
                'access_token',
                'expires_before',
                'refresh_token',
                'refresh_expires_before'
            ])

        logout(self.request)

        if settings.LOGOUT_REDIRECT_URL:
            return resolve_url(settings.LOGOUT_REDIRECT_URL)

        return reverse('keycloak_login')


class SessionIframe(TemplateView):
    template_name = 'django_keycloak/session_iframe.html'

    @property
    def op_location(self):
        realm = self.request.realm
        if realm.server.internal_url:
            return realm.well_known_oidc['check_session_iframe'].replace(
                realm.server.internal_url,
                realm.server.url,
                1
            )
        return realm.server.url

    @property
    def client_id(self):
        if not hasattr(self.request, 'realm'):
            return None

        realm = self.request.realm
        return realm.client.client_id

    def get_context_data(self, **kwargs):
        return super(SessionIframe, self).get_context_data(
            client_id=self.client_id,
            identity_server=self.request.realm.server.url,
            op_location=self.op_location,
            cookie_name=getattr(settings, 'KEYCLOAK_SESSION_STATE_COOKIE_NAME',
                                'session_state')
        )



from django.apps.registry import apps
from django.utils.text import slugify

from keycloak.exceptions import KeycloakClientError

import django_keycloak.services.client


def synchronize_client(client):
    """
    Synchronize all models as resources for a client.

    :type client: django_keycloak.models.Client
    """
    for app_config in apps.get_app_configs():
        synchronize_resources(
            client=client,
            app_config=app_config
        )


def synchronize_resources(client, app_config):
    """
    Synchronize all resources (models) to the Keycloak server for given client
    and Django App.

    :type client: django_keycloak.models.Client
    :type app_config: django.apps.config.AppConfig
    """

    if not app_config.models_module:
        return

    uma1_client = client.uma1_api_client

    access_token = django_keycloak.services.client.get_access_token(
        client=client
    )

    for klass in app_config.get_models():
        scopes = _get_all_permissions(klass._meta)

        try:
            uma1_client.resource_set_create(
                token=access_token,
                name=klass._meta.label_lower,
                type='urn:{client}:resources:{model}'.format(
                    client=slugify(client.client_id),
                    model=klass._meta.label_lower
                ),
                scopes=scopes
            )
        except KeycloakClientError as e:
            if e.original_exc.response.status_code != 409:
                raise


def _get_all_permissions(meta):
    """
    :type meta: django.db.models.options.Options
    :rtype: list
    """
    return meta.default_permissions



import os

from setuptools import setup, find_packages

VERSION = '0.1.2-dev1'

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-keycloak',
    version=VERSION,
    long_description=README,
    package_dir={'': 'src'},
    packages=find_packages('src'),
    extras_require={
        'dev': [
            'bumpversion==0.5.3',
            'twine',
        ],
        'doc': [
            'Sphinx==1.4.4',
            'sphinx-autobuild==0.6.0',
        ]
    },
    setup_requires=[
        'pytest-runner',
        'python-keycloak-client',
    ],
    install_requires=[
        'python-keycloak-client>=0.2.2',
        'Django>=1.11',
    ],
    tests_require=[
        'pytest-django',
        'pytest-cov',
        'mock>=2.0',
        'factory-boy',
        'freezegun'
    ],
    url='https://github.com/Peter-Slump/django-keycloak',
    license='MIT',
    author='Peter Slump',
    author_email='peter@yarf.nl',
    description='Install Django Keycloak.',
    classifiers=[]

)
